@@ -96,8 +96,8 @@ def test_enum_message_type():
         if descriptor.enum is not None:
             if descriptor.enum == GenericOnOff and descriptor.message_type == MetricKind.BINARY_SENSOR:
                 continue
-            if descriptor.message_type not in [get_metric_kind().SELECT, get_metric_kind().SENSOR, get_metric_kind().SWITCH]:
-                errors.append(f"Topic '{descriptor.topic}' has 'enum' but message_type is {descriptor.message_type} (should be SELECT, SENSOR, SWITCH, or BINARY_SENSOR for GenericOnOff)")
+            if descriptor.message_type not in [MetricKind.SELECT, MetricKind.SENSOR, MetricKind.SWITCH, MetricKind.BUTTON]:
+                errors.append(f"Topic '{descriptor.topic}' has 'enum' but message_type is {descriptor.message_type} (should be SELECT, SENSOR, SWITCH, BUTTON, or BINARY_SENSOR for GenericOnOff)")
     if errors:
         pytest.fail("\n".join(errors))
 
@@ -151,16 +151,6 @@ def test_metric_nature_for_energy_and_power():
     if errors:
         pytest.fail("\n".join(errors))
 
-def test_min_max_only_for_number():
-    topics = get_topics()
-    MetricKind = get_metric_kind()
-    errors = []
-    for descriptor in topics:
-        if (descriptor.min is not None or descriptor.max is not None) and descriptor.message_type != MetricKind.NUMBER:
-            errors.append(f"Topic '{descriptor.topic}' has min/max values but message_type is {descriptor.message_type} (should be NUMBER)")
-    if errors:
-        pytest.fail("\n".join(errors))
-
 def test_topic_pattern_structure():
     topics = get_topics()
     MetricKind = get_metric_kind()
@@ -174,13 +164,13 @@ def test_topic_pattern_structure():
                 errors.append(f"Topic '{topic}' has invalid structure (too few parts)")
             continue
         topic_parts = topic.split('/')
-        if len(topic_parts) < 4:
+        if len(topic_parts) < 3:
             errors.append(f"Topic '{topic}' has invalid structure (too few parts)")
-        if descriptor.message_type != MetricKind.SERVICE and not topic.startswith('N/{installation_id}/'):
+        if descriptor.message_type != MetricKind.SERVICE and descriptor.message_type != MetricKind.BUTTON and not topic.startswith('N/{installation_id}/'):
             errors.append(f"Topic '{topic}' must start with 'N{{installation_id}}/'")
         elif descriptor.message_type == MetricKind.SERVICE and not topic.startswith('W/{installation_id}/'):
             errors.append(f"Service topic '{topic}' must start with 'W{{installation_id}}/'")
-        if not topic.find('N/{device_id}/') == -1:
+        if not topic.find('N/{device_id}/') == -1 and len(topic_parts) > 3:
             errors.append(f"Topic '{topic}' must include 'N{{device_id}}/'")
     if errors:
         pytest.fail("\n".join(errors))
@@ -240,6 +230,8 @@ def test_valid_device_type_in_topic():
             if actual_device_type not in valid_device_types:
                 errors.append(f"Formula topic '{descriptor.topic}' has invalid actual_device_type '{actual_device_type}' not defined in DeviceType")
             continue
+        if len(topic_parts) == 3: # root topic without device_type
+            continue
         device_type = topic_parts[2]
         if device_type != "settings" and device_type not in valid_device_types:
             errors.append(f"Topic '{descriptor.topic}' has invalid device_type '{device_type}' not defined in DeviceType")
