@@ -31,13 +31,20 @@ class DeviceType(VictronDeviceEnum):
     RELAY = ("Relay", "<Not used>", "system") # Should be mapped to SYSTEM
     PLATFORM = ("platform", "Platform", "system") # For whatever reason some system topics are under platform
     HEATPUMP = ("heatpump", "Heat Pump")
+    DYNAMIC_ESS = ("DynamicEss", "Dynamic ESS", "system") # Dynamic ESS settings are under system
+    ACLOAD = ("acload", "AC Load")
 
 
 class GenericOnOff(VictronEnum):
-    """On/Off  Enum"""
+    """On/Off Enum"""
     Off = (0, "Off")
     On = (1, "On")
 
+class ChargerMode(VictronEnum):
+    """Charger Mode Enum"""
+    On = (1, "On")
+    Off = (4, "Off")
+
 class InverterMode(VictronEnum):
     """Inverter Mode Enum"""
     ChargerOnly = (1, "Charger Only")
@@ -49,7 +56,7 @@ class PhoenixInverterMode(VictronEnum):
     """Inverter Mode Enum"""
     INVERTER = (2, "Inverter")
     OFF = (4, "Off")
-    ECO = (4, "Eco")
+    ECO = (5, "Eco")
 
 class State(VictronEnum):
     """State Enum"""
@@ -188,9 +195,17 @@ class DESSErrorCode(VictronEnum):
     SOC_LOW = (4, "SOC low")
     BATTRY_CAPACITY_NOT_CONFIGURED = (5,"Battery Capacity Not Configured")
 
+class DESSMode(VictronEnum):
+    """DESS Mode Enum"""
+    OFF = (0, "Off")
+    AUTO_VRM = (1, "Auto / VRM")
+    BUY = (2, "Buy")
+    SELL = (3, "Sell")
+    NODE_RED = (4, "Node-RED")
+
 class DESSRestrictions(VictronEnum):
     NO_RESTRICTIONS = (0, "No Restrictions between battery and the grid")
-    GRID_TO_BATTERY_RESTRICTED = (1, "Grid to battey energy flow restricted")
+    GRID_TO_BATTERY_RESTRICTED = (1, "Grid to battery energy flow restricted")
     BATTERY_TO_GRID_RESTRICTED = (2, "Battery to grid energy flow restricted")
     NO_FLOW = (3, "No energy flow between battery and grid")
 
@@ -280,3 +295,31 @@ class AcActiveInputSource(VictronEnum):
     Generator = (2, "Generator")
     ShorePower = (3, "Shore power")
     NotConnected = (240, "Not connected")
+
+class ChargeSchedule(VictronEnum):
+    """Charge Schedule Enum"""
+    DisabledSunday= (-10, "Disabled (Sunday)")
+    DisabledWeekend= (-9, "Disabled (Weekends)")
+    DisabledWeekdays= (-8, "Disabled (Weekdays)")
+    DisabledEveryDay= (-7, "Disabled (Every day)")
+    DisabledSaturday= (-6, "Disabled (Saturday)")
+    DisabledFriday= (-5, "Disabled (Friday)")
+    DisabledThursday= (-4, "Disabled (Thursday)")
+    DisabledWednesday= (-3, "Disabled (Wednesday)")
+    DisabledTuesday= (-2, "Disabled (Tuesday)")
+    DisabledMonday= (-1, "Disabled (Monday)")
+    Sunday = (0, "Sunday")
+    Monday = (1, "Monday")
+    Tuesday = (2, "Tuesday")
+    Wednesday = (3, "Wednesday")
+    Thursday = (4, "Thursday")
+    Friday = (5, "Friday")
+    Saturday = (6, "Saturday")
+    EveryDay = (7, "Every day")
+    Weekdays = (8, "Weekdays")
+    Weekends = (9, "Weekends")
+
+class ActiveInputEnum(VictronEnum):
+    AC_INPUT_1 = (0, "AC Input 1")
+    AC_INPUT_2 = (1, "AC Input 2")
+    DISCONNECTED = (240, "Disconnected")
