@@ -1,7 +1,7 @@
 """Constants for the victron venus OS client."""
 
 from enum import Enum
-from typing import NamedTuple, Self
+from typing import Self
 
 from dataclasses import dataclass
 
@@ -18,6 +18,8 @@ class MetricKind(Enum):
     SELECT = "select"
     NUMBER = "number"
     SERVICE = "service"
+    BUTTON = "button"
+    TIME = "time"
 
 
 class MetricNature(Enum):
@@ -54,7 +56,6 @@ class MetricType(Enum):
 class ValueType(Enum):
     """Value types."""
 
-    BOOL = "bool"
     INT = "int"
     INT_DEFAULT_0 = "int_0"
     FLOAT = "float"
@@ -62,6 +63,7 @@ class ValueType(Enum):
     ENUM = "enum"
     EPOCH = "epoch"
     INT_SECONDS_TO_HOURS = "int_seconds_to_hours"
+    INT_SECONDS_TO_MINUTES = "int_seconds_to_minutes"
 
 class RangeType(Enum):
     """Range types for numeric values."""
