@@ -9,14 +9,14 @@ import logging
 from typing import TYPE_CHECKING, Callable
 import copy
 
-
 if TYPE_CHECKING:
     from .hub import Hub
 
-from ._unwrappers import VALUE_TYPE_UNWRAPPER, unwrap_bool, unwrap_enum, unwrap_float, unwrap_int_seconds_to_hours
+from ._unwrappers import VALUE_TYPE_UNWRAPPER, unwrap_bool, unwrap_enum, unwrap_float, unwrap_int_seconds_to_hours, unwrap_int_seconds_to_minutes
 from .constants import MetricKind, RangeType
 from .metric import Metric
 from .formula_metric import FormulaMetric
+from .writable_formula_metric import WritableFormulaMetric
 from ._victron_enums import DeviceType
 from .writable_metric import WritableMetric
 from .data_classes import ParsedTopic, TopicDescriptor
@@ -26,11 +26,10 @@ _LOGGER = logging.getLogger(__name__)
 class Device:
     """Class to represent a Victron device."""
 
-    def __init__(self, full_unique_id: str, short_unique_id: str, parsed_topic: ParsedTopic, descriptor: TopicDescriptor) -> None:
+    def __init__(self, unique_id: str, parsed_topic: ParsedTopic, descriptor: TopicDescriptor) -> None:
         """Initialize."""
         self._descriptor = descriptor
-        self._full_unique_id = full_unique_id
-        self._short_unique_id = short_unique_id
+        self._unique_id = unique_id
         self._metrics: dict[str, Metric] = {}
         self._device_type = parsed_topic.device_type
         self._device_id = parsed_topic.device_id
@@ -41,13 +40,12 @@ class Device:
         self._firmware_version = None
         self._custom_name = None
 
-        _LOGGER.debug("Device %s initialized", self._full_unique_id)
+        _LOGGER.debug("Device %s initialized", self._unique_id)
 
     def __repr__(self) -> str:
         """Return a string representation of the device."""
         return (
-            f"Device(full_unique_id={self._full_unique_id}, "
-            f"short_unique_id={self._short_unique_id}, "
+            f"Device(full_unique_id={self.unique_id}, "
             f"name={self.name}, "
             f"model={self.model}, "
             f"manufacturer={self.manufacturer}, "
@@ -100,6 +98,7 @@ class Device:
             self._set_device_property_from_topic(parsed_topic, topic_desc, payload)
             return None
 
+        parsed_topic.finalize_topic_fields(topic_desc)
         if fallback_to_metric_topic:
             value = unwrap_bool(payload)
             if value is None:
@@ -108,25 +107,22 @@ class Device:
                     self.unique_id, topic_desc.short_id
                 )
                 return None
-            return FallbackPlaceholder(device=self, metric_id=topic_desc.short_id, parsed_topic=parsed_topic, topic_descriptor=topic_desc, value=value)
+            return FallbackPlaceholder(device=self, parsed_topic=parsed_topic, topic_descriptor=topic_desc, value=value)
         else:
             value = Device._unwrap_payload(topic_desc, payload)
             if value is None:
                 log_debug(
-                    "Ignoring null fallback_to_metric_topic value for device %s metric %s", 
+                    "Ignoring null topic value for device %s metric %s", 
                     self.unique_id, topic_desc.short_id
                 )
                 return None
 
-        parsed_topic.finalize_topic_fields(topic_desc)
-        short_id = parsed_topic.short_id
-        metric_id = f"{self.unique_id}_{short_id}"
-        metric = self._metrics.get(metric_id)
+        metric = self._metrics.get(parsed_topic.short_id)
         if metric:
-            metric._handle_message(value, event_loop, log_debug)
+            metric._handle_message(value, log_debug)
             return None
         assert value is not None, f"Value must not be None. topic={topic}, payload={payload}"
-        return MetricPlaceholder(self, metric_id, parsed_topic, topic_desc, payload, value)
+        return MetricPlaceholder(self, parsed_topic, topic_desc, payload, value)
 
     @staticmethod
     def _unwrap_payload(topic_desc: TopicDescriptor, payload: str) -> str | float | int | bool | type[Enum] | None:
@@ -134,7 +130,7 @@ class Device:
         unwrapper = VALUE_TYPE_UNWRAPPER[topic_desc.value_type]
         if unwrapper == unwrap_enum:
             return unwrapper(payload, topic_desc.enum)
-        elif unwrapper in [unwrap_float, unwrap_int_seconds_to_hours]:
+        elif unwrapper in [unwrap_float, unwrap_int_seconds_to_hours, unwrap_int_seconds_to_minutes]:
             return unwrapper(payload, topic_desc.precision)
         else:
             return unwrapper(payload)
@@ -144,7 +140,7 @@ class Device:
         """Check if two topics are the same, considering adjustable suffixes."""
         return topic.rsplit('/', 1)[0] == adjustable_topic.rsplit('/', 1)[0]
 
-    def add_placeholder(self, metric_placeholder: MetricPlaceholder, fallback_placeholders: list[FallbackPlaceholder], hub: Hub) -> Metric:
+    def _create_metric_from_placeholder(self, metric_placeholder: MetricPlaceholder, fallback_placeholders: list[FallbackPlaceholder], hub: Hub) -> Metric:
         _LOGGER.info("Creating new metric on device: %s", metric_placeholder)
 
         new_topic_desc = metric_placeholder.topic_descriptor
@@ -178,24 +174,29 @@ class Device:
         name = ParsedTopic.replace_ids(metric_placeholder.topic_descriptor.name, metric_placeholder.parsed_topic.key_values)
         assert metric_placeholder.parsed_topic.device_type is not None, "device_type must be set for metric"
 
-        if new_topic_desc.message_type in [MetricKind.SWITCH, MetricKind.NUMBER, MetricKind.SELECT]:
-            metric = WritableMetric(self, metric_placeholder.metric_id, name, new_topic_desc, metric_placeholder.parsed_topic, hub)
+        if new_topic_desc.message_type in [MetricKind.SWITCH, MetricKind.NUMBER, MetricKind.SELECT, MetricKind.BUTTON, MetricKind.TIME]:
+            metric = WritableMetric(device = self, name = name, descriptor = new_topic_desc, unique_id = metric_placeholder.parsed_topic.unique_id, short_id = metric_placeholder.parsed_topic.short_id, key_values = metric_placeholder.parsed_topic.key_values, topic = metric_placeholder.parsed_topic.full_topic, hub = hub)
         else:
-            metric = Metric(self, metric_placeholder.metric_id, name, new_topic_desc, metric_placeholder.parsed_topic.short_id, metric_placeholder.parsed_topic.key_values, hub)
-        metric._handle_message(metric_placeholder.value, None, _LOGGER.debug)
-        self._metrics[metric.unique_id] = metric
+            metric = Metric(device = self, name = name, descriptor = new_topic_desc, unique_id = metric_placeholder.parsed_topic.unique_id, short_id = metric_placeholder.parsed_topic.short_id, key_values = metric_placeholder.parsed_topic.key_values,hub = hub)
+        metric._handle_message(metric_placeholder.value, _LOGGER.debug)
+        self._metrics[metric.short_id] = metric
         return metric
 
-    def add_formula_metric(self, topic_desc: TopicDescriptor, hub: Hub) -> FormulaMetric:
+    def _add_formula_metric(self, topic_desc: TopicDescriptor, hub: Hub, key_values: dict[str, str]) -> FormulaMetric:
         assert topic_desc.name is not None, "name must be set for topic"
-        metric_id = f"{self.unique_id}_{topic_desc.short_id}"
-        metric = FormulaMetric(self, metric_id, topic_desc.name, topic_desc, hub)
-        self._metrics[metric.unique_id] = metric
+        name = ParsedTopic.replace_ids(topic_desc.name, key_values)
+        short_id = ParsedTopic.replace_ids(topic_desc.short_id, key_values)
+        unique_id = ParsedTopic.make_unique_id(self.unique_id, short_id)
+        if topic_desc.message_type in [MetricKind.SWITCH, MetricKind.NUMBER, MetricKind.SELECT, MetricKind.BUTTON, MetricKind.TIME]:
+            metric = WritableFormulaMetric(device = self, name = name, descriptor = topic_desc, unique_id = unique_id, short_id = short_id, hub = hub, key_values = key_values)
+        else:
+            metric = FormulaMetric(device = self, name = name, descriptor = topic_desc, unique_id = unique_id, short_id = short_id, hub = hub, key_values = key_values)
+        self._metrics[metric.short_id] = metric
         return metric
 
-    def get_metric_from_unique_id(self, unique_id: str) -> Metric | WritableMetric | None:
+    def get_metric(self, short_id: str) -> Metric | WritableMetric | None:
         """Get a metric from a unique id."""
-        return self._metrics.get(unique_id)
+        return self._metrics.get(short_id)
 
     @property
     def metrics(self) -> list[Metric | WritableMetric]:
@@ -205,12 +206,7 @@ class Device:
     @property
     def unique_id(self) -> str:
         """Return the unique id of the device."""
-        return self._full_unique_id
-
-    @property
-    def short_unique_id(self) -> str:
-        """Return the short unique id of the device."""
-        return self._short_unique_id
+        return self._unique_id
 
     @property
     def name(self) -> str | None:
@@ -264,22 +260,20 @@ class Device:
 @dataclass
 class MetricPlaceholder:
     device: Device
-    metric_id: str
     parsed_topic: ParsedTopic
     topic_descriptor: TopicDescriptor
     payload: str
     value: str | float | int | bool | type[Enum]
 
     def __repr__(self) -> str:
-        return f"MetricPlaceholder(device={self.device}, metric_id={self.metric_id}, parsed_topic={self.parsed_topic}, topic_descriptor={self.topic_descriptor}, payload={self.payload}, value={self.value})"
+        return f"MetricPlaceholder(device={self.device}, parsed_topic={self.parsed_topic}, topic_descriptor={self.topic_descriptor}, payload={self.payload}, value={self.value})"
     
 @dataclass
 class FallbackPlaceholder:
     device: Device
-    metric_id: str
     parsed_topic: ParsedTopic
     topic_descriptor: TopicDescriptor
     value: str | float | int | bool | type[Enum]
 
     def __repr__(self) -> str:
-        return f"FallbackPlaceholder(device={self.device}, metric_id={self.metric_id}, parsed_topic={self.parsed_topic}, topic_descriptor={self.topic_descriptor}, value={self.value})"
+        return f"FallbackPlaceholder(device={self.device}, parsed_topic={self.parsed_topic}, topic_descriptor={self.topic_descriptor}, value={self.value})"
