@@ -1,5 +1,6 @@
 """Module to communicate with the Venus OS MQTT Broker."""
 
+from dataclasses import replace
 import asyncio
 import copy
 import json
@@ -9,6 +10,7 @@ import ssl
 import re
 import string
 from typing import Any, Callable, Optional, Tuple
+import time
 
 import paho.mqtt.client as mqtt
 from paho.mqtt.client import Client as MQTTClient, PayloadType
@@ -27,6 +29,9 @@ from .metric import Metric
 
 _LOGGER = logging.getLogger(__name__)
 CONNECT_MAX_FAILED_ATTEMPTS = 3
+FORCE_INVALIDATE_AFTER_NOT_CONNECTED_SECONDS = 120
+FULL_PUBLISH_MIN_INTERVAL_SECONDS = 45
+MINIMUM_FULLY_SUPPORTED_VERSION = 3.5
 
 # Modify the logger to include instance_id without changing the tracing level
 # class InstanceIDFilter(logging.Filter):
@@ -77,7 +82,7 @@ class Hub:
         topic_log_info: str | None = None,
         operation_mode: OperationMode = OperationMode.FULL,
         device_type_exclude_filter: list[DeviceType] | None = None,
-        update_frequency_seconds: int | None = None
+        update_frequency_seconds: int | None = None,
     ) -> None:
         """
         Initialize a Hub instance for communicating with a Venus OS MQTT broker.
@@ -176,8 +181,8 @@ class Hub:
         if update_frequency_seconds is not None and not isinstance(update_frequency_seconds, int):
             raise TypeError(f"update_frequency_seconds must be an integer or None, got type={type(update_frequency_seconds).__name__}, value={update_frequency_seconds!r}")
         _LOGGER.info(
-            "Initializing Hub[ID: %d](host=%s, port=%d, username=%s, use_ssl=%s, installation_id=%s, model_name=%s, topic_prefix=%s, operation_mode=%s, device_type_exclude_filter=%s, update_frequency_seconds=%s)",
-            self._instance_id, host, port, username, use_ssl, installation_id, model_name, topic_prefix, operation_mode, device_type_exclude_filter, update_frequency_seconds
+            "Initializing Hub[ID: %d](host=%s, port=%d, username=%s, use_ssl=%s, installation_id=%s, model_name=%s, topic_prefix=%s, operation_mode=%s, device_type_exclude_filter=%s, update_frequency_seconds=%s, topic_log_info=%s)",
+            self._instance_id, host, port, username, use_ssl, installation_id, model_name, topic_prefix, operation_mode, device_type_exclude_filter, update_frequency_seconds, topic_log_info,
         )
         self._model_name = model_name
         self.host = host
@@ -195,21 +200,27 @@ class Hub:
         self._snapshot = {}
         self._keepalive_task = None
         self._connected_event = asyncio.Event()
-        self._connected_failed_attempts = 0
         self._on_new_metric: CallbackOnNewMetric | None = None
         self._topic_log_info = topic_log_info
         self._operation_mode = operation_mode
         self._device_type_exclude_filter = device_type_exclude_filter
+        self._update_frequency_seconds = update_frequency_seconds
         # The client ID is generated using a random string and the instance ID. It has to be unique between all clients connected to the same mqtt server. If not, they may reset each other connection.
         random_string = ''.join(random.choices(string.ascii_letters + string.digits, k=8))
         self._client_id = f"victron_mqtt-{random_string}-{self._instance_id}"
         self._keepalive_counter = 0
-        self._metrics_placeholders: list[MetricPlaceholder] = []
-        self._fallback_placeholders: list[FallbackPlaceholder] = []
+        self._metrics_placeholders: dict[str, MetricPlaceholder] = {}
+        self._fallback_placeholders: dict[str, FallbackPlaceholder] = {}
         self._all_metrics: dict[str, Metric] = {}
         self._first_connect = True
         self._first_full_publish = True
-        self._update_frequency_seconds = update_frequency_seconds
+        self._connect_failed_attempts = 0
+        self._connect_failed = False
+        # Track when _handle_full_publish_message was last invoked for our client
+        # Use monotonic time to avoid issues with system clock changes
+        self._last_full_publish_called: float = 0.0
+        self._firmware_version: float = 0.0
+        self._connect_failed_since: float = 0.0
 
         # Filter the active topics
         metrics_active_topics: list[TopicDescriptor] = []
@@ -219,7 +230,7 @@ class Hub:
                 continue
             if topic.message_type == MetricKind.SERVICE:
                 self._service_active_topics[topic.short_id] = topic
-            else:        
+            else:
                 #if operation_mode is READ_ONLY we should change all TopicDescriptor to SENSOR and BINARY_SENSOR
                 if operation_mode != OperationMode.READ_ONLY or topic.message_type in [MetricKind.ATTRIBUTE, MetricKind.SENSOR, MetricKind.BINARY_SENSOR]:
                     metrics_active_topics.append(topic)
@@ -255,6 +266,9 @@ class Hub:
         def build_multi_map(items, key_func):
             result = {}
             for item in items:
+                # No need to index formula topics, they are discovered by _pending_formula_topics
+                if item.is_formula:
+                    continue
                 key = key_func(item)
                 if key in result:
                     existing = result[key]
@@ -282,9 +296,13 @@ class Hub:
         _LOGGER.info("Connecting to MQTT broker at %s:%d", self.host, self.port)
         assert self._client is not None
 
-        if self.username is not None:
+        # Based on https://community.victronenergy.com/t/cerbo-mqtt-webui-network-security-profile-configuration/34112
+        # it seems that Cerbos will not allow you to configure username, only passwords.
+        # still, you do need to put in some username to get the MQTT client work.
+        if self.password is not None:
+            username = "victron_mqtt" if self.username is None else self.username
             _LOGGER.info("Setting auth credentials for user: %s", self.username)
-            self._client.username_pw_set(self.username, self.password)
+            self._client.username_pw_set(username, self.password)
         
         if self.use_ssl:
             _LOGGER.info("Setting up SSL context")
@@ -292,15 +310,18 @@ class Hub:
             ssl_context.check_hostname = False
             ssl_context.verify_mode = ssl.VerifyMode.CERT_NONE
             self._client.tls_set_context(ssl_context)
-            
+        
         self._client.on_connect = self._on_connect
         self._client._on_disconnect = self._on_disconnect
         self._client.on_message = self._on_message
         self._client.on_connect_fail = self.on_connect_fail
         #self._client.on_log = self._on_log
-        self._connected_failed_attempts = 0
+        self._connect_failed_attempts = 0
+        self._connect_failed_since = 0.0
+        self._connect_failed = False
         self._loop = asyncio.get_event_loop()
         #self._loop.set_task_factory(lambda loop, coro: TracedTask(coro, loop=loop, name=name))
+        self._last_full_publish_called = time.monotonic() # Initialize last full publish time
 
         try:
             _LOGGER.info("Starting paho mqtt")
@@ -309,9 +330,10 @@ class Hub:
             self._client.connect_async(self.host, self.port)
             _LOGGER.info("Waiting for connection event")
             await self._wait_for_connect()
-            if self._connected_failed_attempts >= CONNECT_MAX_FAILED_ATTEMPTS:
-                _LOGGER.error("Failed to connect to MQTT broker")
-                raise CannotConnectError("Failed to connect to MQTT broker")
+            if self._client is None:
+                raise CannotConnectError("Got request to disconnect while still connecting")
+            if self._connect_failed:
+                raise CannotConnectError(f"Failed to connect to MQTT broker: {self.host}:{self.port}")
             _LOGGER.info("Successfully connected to MQTT broker at %s:%d", self.host, self.port)
             if self._installation_id is None:
                 _LOGGER.info("No installation ID provided, attempting to read from device")
@@ -328,8 +350,8 @@ class Hub:
             self._start_keep_alive_loop()
             _LOGGER.info("Connected. Installation ID: %s", self._installation_id)
         except Exception as exc:
-            _LOGGER.error("Failed to connect to MQTT broker: %s", exc, exc_info=True)
-            raise CannotConnectError(f"Failed to connect to MQTT broker: {exc}") from exc
+            _LOGGER.error("Failed to connect: %s", exc, exc_info=True)
+            raise CannotConnectError(f"Failed to connect: {exc}") from exc
 
     def publish(self, topic_short_id: str, device_id: str, value: str | float | int | None) -> None:
         """Publish a message to the MQTT broker."""
@@ -355,6 +377,7 @@ class Hub:
 
     def _on_connect_internal(self, client: MQTTClient, userdata: Any, flags: dict, rc: int, properties: Optional[dict] = None) -> None:
         """Handle connection callback."""
+        self._connect_failed_since = 0
         if self._client is None:
             _LOGGER.warning("Got new connection while self._client is None, ignoring")
             return
@@ -402,19 +425,43 @@ class Hub:
 
         self._handle_normal_message(topic, payload, log_debug)
 
-    def is_dependency_met(self, topic: TopicDescriptor, relevant_device: Device) -> Tuple[bool, list[Metric]]:
+        # Ensure _handle_full_publish_message runs at least once every interval.
+        # This is to handle old cerbo versions that do not send full publish completed messages.
+        # Issue #139 and #205
+        now = time.monotonic()
+        # If we've never called it or it's been longer than the configured interval,
+        # call _handle_full_publish_message with an empty payload to trigger periodic handling.
+        if now - self._last_full_publish_called >= FULL_PUBLISH_MIN_INTERVAL_SECONDS:
+            _LOGGER.debug("Periodic trigger: calling _handle_full_publish_message (last %.1fs ago)", now - self._last_full_publish_called)
+            # Use an empty JSON object.
+            self._handle_full_publish_message("{}")
+
+    def is_formula_dependency_met(self, topic: TopicDescriptor, relevant_device: Device, key_values: dict[str, str]) -> Tuple[bool, list[Metric]]:
         if not topic.depends_on:
             return True, []
         dependencies: list[Metric] = []
         for dependency in topic.depends_on:
-            metric_name = f"{relevant_device.short_unique_id}_{dependency}"
-            dependency_metric = self._all_metrics.get(metric_name)
+            metric_unique_id = ParsedTopic.make_unique_id(relevant_device.unique_id, dependency)
+            metric_unique_id = ParsedTopic.replace_ids(metric_unique_id, key_values)
+            dependency_metric = self._all_metrics.get(metric_unique_id)
             if dependency_metric is None:
-                _LOGGER.debug("Formula topic %s is missing dependency metric: %s", topic.topic, metric_name)
+                _LOGGER.debug("Formula topic %s is missing dependency metric: %s", topic.topic, metric_unique_id)
                 return False, []
             dependencies.append(dependency_metric)
         return True, dependencies
 
+    def is_regular_dependency_met(self, metric_placeholder: MetricPlaceholder) -> bool:
+        if not metric_placeholder.topic_descriptor.depends_on:
+            return True
+        for dependency in metric_placeholder.topic_descriptor.depends_on:
+            metric_id = ParsedTopic.replace_ids(dependency, metric_placeholder.parsed_topic.key_values)
+            dependency_metric = self._all_metrics.get(metric_id)
+            dependency_placeholder = self._metrics_placeholders.get(metric_id)
+            if dependency_metric is None and dependency_placeholder is None:
+                _LOGGER.debug("Regular topic %s is missing dependency metric: %s", metric_placeholder.parsed_topic.full_topic, metric_id)
+                return False
+        return True
+
     def _handle_full_publish_message(self, payload: str) -> None:
         """Handle full publish message."""
         echo = self.get_keepalive_echo(payload)
@@ -428,38 +475,51 @@ class Hub:
         if echo and not echo.startswith(self._client_id):
             _LOGGER.debug("Not our echo: %s", echo)
             return
+        # Update the last-called timestamp when we handled a full-publish for our client
+        self._last_full_publish_called = time.monotonic()
         
         _LOGGER.debug("Full publish completed: %s", echo)
         new_metrics: list[tuple[Device, Metric]] = []
-        for metric_placeholder in self._metrics_placeholders:
-            metric = metric_placeholder.device.add_placeholder(metric_placeholder, self._fallback_placeholders, self)
-            self._all_metrics[metric._hub_unique_id] = metric
-            new_metrics.append((metric_placeholder.device, metric))
+        if len(self._metrics_placeholders) > 0:
+            fallback_placeholders_list = list(self._fallback_placeholders.values())
+            for metric_placeholder in self._metrics_placeholders.values():
+                # Check if depedency met, if not, the topic will get ignored
+                if not self.is_regular_dependency_met(metric_placeholder):
+                    continue
+                metric = metric_placeholder.device._create_metric_from_placeholder(metric_placeholder, fallback_placeholders_list, self)
+                self._all_metrics[metric.unique_id] = metric
+                new_metrics.append((metric_placeholder.device, metric))
         self._metrics_placeholders.clear()
         self._fallback_placeholders.clear()
         # Activate formula entities
         if len(new_metrics) > 0:
-            new_formula_metrics: list[tuple[Device, Metric]] = []
             for topic in self._pending_formula_topics:
                 _LOGGER.debug("Trying to resolve formula topic: %s", topic)
                 relevant_devices: list[Device] = [device for device in self._devices.values() if device.device_type.code == topic.topic.split("/")[1]]
                 for device in relevant_devices:
-                    is_met, dependencies = self.is_dependency_met(topic, device)
-                    if is_met:
+                    # We need all depends_on metric to get the key_values associated with it to be able to generate new metrics per moniker.
+                    all_new_metrics_with_same_depends_on_generic_id = [t[1] for t in new_metrics if t[1]._device == device and t[1].generic_short_id == topic.depends_on[0]]
+                    for depends_on_metric in all_new_metrics_with_same_depends_on_generic_id:
+                        metric_unique_id = ParsedTopic.make_unique_id(device.unique_id, topic.short_id)
+                        metric_unique_id = ParsedTopic.replace_ids(metric_unique_id, depends_on_metric.key_values)
+                        if self._all_metrics.get(metric_unique_id) is not None:
+                            continue
+                        is_met, dependencies = self.is_formula_dependency_met(topic, device, depends_on_metric.key_values)
+                        if not is_met:
+                            continue
                         _LOGGER.info("Formula topic resolved: %s", topic)
-                        metric = device.add_formula_metric(topic, self)
+                        metric = device._add_formula_metric(topic, self, depends_on_metric.key_values)
                         depends_on: dict[str, Metric] = {}
                         for dependency_metric in dependencies:
                             dependency_metric.add_dependency(metric)
-                            depends_on[dependency_metric._hub_unique_id] = dependency_metric
-                        metric.init(depends_on, self._loop, _LOGGER.debug)
+                            depends_on[dependency_metric.unique_id] = dependency_metric
+                        metric.init(depends_on, _LOGGER.debug)
                         _LOGGER.info("Formula metric created: %s", metric)
-                        new_formula_metrics.append((device, metric))
-            # Send all new metrics to clients
-            new_metrics.extend(new_formula_metrics)
+                        self._all_metrics[metric.unique_id] = metric
+                        new_metrics.append((device, metric))
         # We are sending the new metrics now as we can be sure that the metric handled all the attribute topics and now ready.
         for device, metric in new_metrics:
-            metric.phase2_init(device.short_unique_id, self._all_metrics)
+            metric.phase2_init(device.unique_id, self._all_metrics)
             try:
                 if callable(self._on_new_metric):
                     if self._loop.is_running():
@@ -478,9 +538,9 @@ class Hub:
                         ver_str = version_metric.value[1:]
                         if "~" in ver_str:
                             ver_str = ver_str.split("~", 1)[0]
-                        firmware_version = float(ver_str)
-                        if firmware_version < 3.5:
-                            _LOGGER.warning("Firmware version is below v3.5: %s", version_metric.value)
+                        self._firmware_version = float(ver_str)
+                        if self._firmware_version < MINIMUM_FULLY_SUPPORTED_VERSION:
+                            _LOGGER.warning("Firmware version is below v3.5: %s. Reduced functionality may occur.", version_metric.value)
                         else:
                             _LOGGER.info("Firmware version is good enough: %s", version_metric.value)
                     except (ValueError, TypeError):
@@ -492,7 +552,8 @@ class Hub:
         if self._loop.is_running():
             self._loop.call_soon_threadsafe(self._first_refresh_event.set)
         self._first_full_publish = False
-
+        _LOGGER.debug("Full publish handling completed")
+        
     def _handle_installation_id_message(self, topic: str) -> None:
         """Handle installation ID message."""
         parsed_topic = ParsedTopic.from_topic(topic)
@@ -534,11 +595,17 @@ class Hub:
                 return
 
         device = self._get_or_create_device(parsed_topic, desc)
-        metric_place_holder = device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload, self._loop, log_debug, self)
-        if isinstance(metric_place_holder, MetricPlaceholder):
-            self._metrics_placeholders.append(metric_place_holder)
-        elif isinstance(metric_place_holder, FallbackPlaceholder):
-            self._fallback_placeholders.append(metric_place_holder)
+        placeholder = device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload, self._loop, log_debug, self)
+        if isinstance(placeholder, MetricPlaceholder):
+            existing_placeholder = self._metrics_placeholders.get(placeholder.parsed_topic.unique_id)
+            if existing_placeholder:
+                log_debug("Replacing existing metric placeholder: %s", existing_placeholder)
+            self._metrics_placeholders[placeholder.parsed_topic.unique_id] = placeholder
+        elif isinstance(placeholder, FallbackPlaceholder):
+            existing_placeholder = self._fallback_placeholders.get(placeholder.parsed_topic.unique_id)
+            if existing_placeholder:
+                log_debug("Replacing existing fallback placeholder: %s", existing_placeholder)
+            self._fallback_placeholders[placeholder.parsed_topic.unique_id] = placeholder
 
     async def disconnect(self) -> None:
         """Disconnect from the hub."""
@@ -574,8 +641,12 @@ class Hub:
 
     def _publish(self, topic: str, value: PayloadType) -> None:
         assert self._client is not None
+        # Determine log level based on the substring
+        is_info_level = self._topic_log_info and self._topic_log_info in topic
+        log_debug = _LOGGER.info if is_info_level else _LOGGER.debug
+
         prefixed_topic = self._add_topic_prefix(topic)
-        _LOGGER.debug("Publishing message to topic: %s, value: %s", prefixed_topic, value)
+        log_debug("Publishing message to topic: %s, value: %s", prefixed_topic, value)
         self._client.publish(prefixed_topic, value)
 
     async def _keepalive_loop(self) -> None:
@@ -588,7 +659,8 @@ class Hub:
                 await asyncio.sleep(30)
                 # We should keep alive all metrics every 60 seconds
                 count += 1
-                if count % 2 == 0:
+                # Old firmwars dont resend values after the keepalive message so we cant use this logic of invalidation if there is no new value
+                if self._firmware_version >= MINIMUM_FULLY_SUPPORTED_VERSION and count % 2 == 0:
                     self._keepalive_metrics()
             except asyncio.CancelledError:
                 _LOGGER.info("Keepalive loop canceled")
@@ -597,11 +669,15 @@ class Hub:
                 _LOGGER.error("Error in keepalive loop: %s", exc, exc_info=True)
                 await asyncio.sleep(5)  # Short delay before retrying
 
-    def _keepalive_metrics(self) -> None:
+    def _keepalive_metrics(self, force_invalidate: bool = False) -> None:
         """Keep alive all metrics."""
         _LOGGER.debug("Keeping alive all metrics")
         for metric in self._all_metrics.values():
-            metric._keepalive(self._loop, _LOGGER.debug)
+            # Determine log level based on the substring
+            is_info_level = self._topic_log_info and self._topic_log_info in metric._descriptor.topic
+            log_debug = _LOGGER.info if is_info_level else _LOGGER.debug
+
+            metric._keepalive(force_invalidate, log_debug)
 
     def _start_keep_alive_loop(self) -> None:
         """Start the keep_alive loop."""
@@ -778,7 +854,7 @@ class Hub:
     async def _wait_for_connect(self) -> None:
         """Wait for the first connection to complete."""
         try:
-            await asyncio.wait_for(self._connected_event.wait(), timeout=25)
+            await asyncio.wait_for(self._connected_event.wait(), timeout=25) # 25 seconds
         except asyncio.TimeoutError:
             _LOGGER.error("Timeout waiting for first first connection")
             raise CannotConnectError("Timeout waiting for first connection")
@@ -795,42 +871,28 @@ class Hub:
 
     def _get_or_create_device(self, parsed_topic: ParsedTopic, desc: TopicDescriptor) -> Device:
         """Get or create a device based on topic."""
-        full_unique_id, short_unique_id = self._create_device_unique_id(
-            parsed_topic.installation_id,
-            parsed_topic.device_type.code,
-            parsed_topic.device_id,
-        )
-        device = self._devices.get(short_unique_id)
+        device_unique_id = parsed_topic.get_device_unique_id()
+        device = self._devices.get(device_unique_id)
         if device is None:
-            _LOGGER.info("Creating new device: unique_id=%s, parsed_topic=%s", full_unique_id, parsed_topic)
+            _LOGGER.info("Creating new device: unique_id=%s, parsed_topic=%s", device_unique_id, parsed_topic)
             device = Device(
-                full_unique_id,
-                short_unique_id,
+                device_unique_id,
                 parsed_topic,
                 desc,
             )
-            self._devices[short_unique_id] = device
+            self._devices[device_unique_id] = device
         return device
 
-    def _create_device_unique_id(self, installation_id: str, device_type: str, device_id: str) -> tuple[str, str]:
-        """Create a unique ID for a device."""
-        short_unique_id = f"{device_type}_{device_id}"
-        full_unique_id = f"{installation_id}_{short_unique_id}"
-        return full_unique_id, short_unique_id
-
-    def _get_device_unique_id_from_metric_unique_id(self, unique_id: str) -> str:
-        return "_".join(unique_id.split("_")[:3])
-
-    def get_metric_from_unique_id(self, unique_id: str) -> Metric | None:
+    def get_metric(self, unique_id: str) -> Metric | None:
         """Get a metric from a unique id."""
-        id_without_installation = unique_id.split("_", 1)[-1]
-        metric = self._all_metrics.get(id_without_installation)
+        metric = self._all_metrics.get(unique_id)
         return metric
 
     @property
     def devices(self) -> dict[str, Device]:
         "Return a list of devices attached to the hub. Requires initialize_devices_and_metrics() to be called first."
-        return dict(self._devices)
+        # Return only devices with at least a single metric
+        return {k: v for k, v in self._devices.items() if v.metrics}
 
     @property
     def installation_id(self) -> str | None:
@@ -857,10 +919,20 @@ class Hub:
     def on_connect_fail(self, client: MQTTClient, userdata: Any) -> None:
         """Handle connection failure callback."""
         _LOGGER.warning("Connection to MQTT broker failed")
-        self._connected_failed_attempts += 1
-        if self._connected_failed_attempts >= CONNECT_MAX_FAILED_ATTEMPTS:
+        if self._connect_failed_since == 0:
+            self._connect_failed_since = time.monotonic()
+        self._connect_failed_attempts += 1
+        # Check if we have reached the maximum number of failed attempts
+        if self._connect_failed_attempts >= CONNECT_MAX_FAILED_ATTEMPTS:
+            self._connect_failed = True
             if self._loop.is_running():
                 self._loop.call_soon_threadsafe(self._connected_event.set)
+        # This code is not really needed in newer firmwares as metrics will get invalidated individually. With older firmwares, we can force invalidation after we are disconnected for enough time
+        disconnected_for = time.monotonic() - self._connect_failed_since
+        if disconnected_for > FORCE_INVALIDATE_AFTER_NOT_CONNECTED_SECONDS:
+            _LOGGER.info("Disconnected for %d seconds. Invalidating all metrics", disconnected_for)
+            self._keepalive_metrics(force_invalidate=True)
+            self._connect_failed_since = 0  # Reset the timer
 
     @staticmethod
     def expand_topic_list(topic_list: list[TopicDescriptor]) -> list[TopicDescriptor]:
@@ -878,12 +950,14 @@ class Hub:
                 match = matches[0]
                 key, start, end = match.group(1), int(match.group(2)), int(match.group(3))
                 for i in range(start, end+1):
-                    new_kwargs = td.__dict__.copy()
-                    new_kwargs['topic'] = pattern.sub(str(i), td.topic)
-                    new_kwargs['key_values'] = {key: str(i)}
-                    expanded.append(TopicDescriptor(**new_kwargs))
+                    expanded.append(replace(
+                        td,
+                        topic=pattern.sub(str(i), td.topic),
+                        key_values={key: str(i)}
+                    ))
             else:
-                expanded.append(td)
+                # Use replace to trigger __post_init__ even for unchanged items
+                expanded.append(replace(td))
         return expanded
 
     @property
@@ -909,9 +983,13 @@ class Hub:
     @staticmethod
     def get_keepalive_echo(value: str) -> str | None:
         """Extract the keepalive echo value from the published message."""
-        publish_completed_message = json.loads(value)
-        echo = publish_completed_message.get("full-publish-completed-echo", None)
-        return echo
+        try:
+            publish_completed_message = json.loads(value)
+            echo = publish_completed_message.get("full-publish-completed-echo", None)
+            return echo
+        except (json.JSONDecodeError, KeyError, ValueError, TypeError):
+            _LOGGER.error("Failed to extract keepalive echo: %s", value)
+            return None
 
 
 class CannotConnectError(Exception):
