@@ -5,13 +5,12 @@ Support for Victron Venus sensors. The sensor itself has no logic,
 
 from __future__ import annotations
 
-import asyncio
 from collections.abc import Callable
 import time
 import logging
 from typing import TYPE_CHECKING, Any
 
-from .id_utils import replace_complex_id_to_simple, replace_complex_ids
+from .id_utils import replace_complex_ids
 from .constants import MetricKind, MetricNature, MetricType
 from .data_classes import ParsedTopic, TopicDescriptor
 
@@ -27,28 +26,34 @@ CallbackOnUpdate = Callable[["Metric", Any], None]
 class Metric:
     """Representation of a Victron Venus sensor."""
 
-    def __init__(self, device: Device, unique_id: str, name: str, descriptor: TopicDescriptor, short_id: str, key_values: dict[str, str], hub: Hub) -> None:
+    def __init__(self, *, device: Device | None = None, name: str | None = None, descriptor: TopicDescriptor | None = None, unique_id: str | None = None, short_id: str | None = None, key_values: dict[str, str] | None = None, hub: Hub | None = None, **kwargs) -> None:
         """Initialize the sensor."""
-        _LOGGER.debug(
-            "Creating new metric: unique_id=%s, type=%s, nature=%s",
-            unique_id, descriptor.metric_type, descriptor.metric_nature
-        )
+        assert device is not None
+        assert name is not None
+        assert descriptor is not None
         assert descriptor.name is not None, "name must be set for metric"
-        self._device = device
-        self._descriptor = descriptor
-        self._unique_id = unique_id
-        self._value = None
-        self._short_id = short_id
-        self._name = name
+        assert unique_id is not None
+        assert key_values is not None
+        assert short_id is not None
+        assert hub is not None
+        _LOGGER.debug(
+            "Creating new metric: short_id=%s, type=%s, nature=%s",
+            short_id, descriptor.metric_type, descriptor.metric_nature
+        )
+        self._device: Device = device
+        self._descriptor: TopicDescriptor = descriptor
+        self._unique_id: str = unique_id
+        self._value: Any = None
+        self._short_id: str = short_id
+        self._name: str  = name
         self._key_values: dict[str, str] = key_values
         self._on_update: CallbackOnUpdate | None = None
-        self._generic_name: str | None = None
-        self._generic_short_id: str | None = None
         self._depend_on_me: list[FormulaMetric] = []
-        self._hub_unique_id: str = f"{device.short_unique_id}_{self._short_id}"
         self._hub = hub
-        self._last_update: float = 0
+        self._last_notified: float = 0
         self._last_seen: float = 0
+        self._generic_short_id = self._descriptor.short_id
+        self._generic_name = self._descriptor.generic_name
 
         _LOGGER.debug("Metric %s initialized", repr(self))
 
@@ -60,9 +65,7 @@ class Metric:
             f"Metric(unique_id={self.unique_id}, "
             f"descriptor={self._descriptor}, "
             f"value={self.value}, "
-            f"generic_short_id={self._generic_short_id}, "
             f"short_id={self._short_id}, "
-            f"generic_name={self._generic_name}, "
             f"name={self._name}, "
             f"{key_values_part})"
             )
@@ -75,8 +78,6 @@ class Metric:
         assert self._descriptor.name is not None, f"name must be set for topic: {self._descriptor.topic}"
         name_temp = ParsedTopic.replace_ids(self._descriptor.name, self._key_values)
         self._name = self._replace_ids(name_temp, device_id, all_metrics)
-        self._generic_name = self._descriptor.generic_name
-        self._generic_short_id = replace_complex_id_to_simple(self._descriptor.short_id)
 
     def _replace_ids(self, orig_str: str, device_id: str, all_metrics: dict[str, Metric]) -> str:
         def replace_match(match):
@@ -105,7 +106,8 @@ class Metric:
         """Returns the formatted value of the metric."""
         if value is None:
             return ""
-
+        if isinstance(value, float) and self._descriptor.precision is not None and self._descriptor.precision == 0:
+            value = int(value)
         if self._descriptor.unit_of_measurement is None:
             return str(value)
         else:
@@ -189,63 +191,72 @@ class Metric:
         """Sets the on_update callback."""
         self._on_update = value
 
-    def _keepalive(self, event_loop: asyncio.AbstractEventLoop | None, log_debug: Callable[..., None]):
+    def _keepalive(self, force_invalidate: bool, log_debug: Callable[..., None]):
         """Reset metrics value if no updates or send last values if they got skipped"""
-        silence_timeout = min(60, self._hub._update_frequency_seconds * 3) if self._hub._update_frequency_seconds is not None else 60
-        now = time.time()
+        silence_timeout = max(60, self._hub._update_frequency_seconds * 3) if self._hub._update_frequency_seconds is not None else 60
+        now = time.monotonic()
         elapsed = now - self._last_seen
-        if elapsed > silence_timeout:
+        if (force_invalidate or elapsed > silence_timeout) and self._value is not None:
             log_debug("Metric %s has been silent for %.2fs, resetting", self.unique_id, elapsed)
-            self._handle_message(None, event_loop, log_debug)
-        elif self._last_seen > self._last_update:
-            log_debug("Metric %s has been updated at %.2f but not published since %.2fs, re-publishing", self.unique_id, self._last_update, self._last_seen)
-            self._handle_message(self._value, event_loop, log_debug)
+            self._handle_message(None, log_debug, update_last_seen=False) #Dont update the last_seen as it wasnt seen
+            return
+        if self._last_seen > self._last_notified:
+            log_debug("Metric %s has been updated at %.2f but not published since %.2fs, re-publishing", self.unique_id, self._last_notified, self._last_seen)
+            self._handle_message(self._value, log_debug, update_last_seen=False, force=True)
+            return
+        log_debug("Metric is active and up-to-date: %s", self.unique_id)
 
-
-    def _handle_message(self, value, event_loop: asyncio.AbstractEventLoop | None, log_debug: Callable[..., None]):
+    def _handle_message(self, value, log_debug: Callable[..., None], update_last_seen: bool = True, force: bool = False):
         """Handle a message."""
-        now = time.time()
-        self._last_seen = now
+        now = time.monotonic()
+        if update_last_seen:
+            self._last_seen = now
+        should_notify = False
 
-        changed = False
-        if value != self._value:
+        # In case of zero update frequency, always consider changed when MQTT message is received
+        # also if this is the first time the metric is being notified
+        if force or self._hub._update_frequency_seconds == 0 or self._last_notified == 0:
+            should_notify = True
+            force = True
+        elif value != self._value:
             log_debug(
                 "Metric %s value changed: %s -> %s %s",
                 self.unique_id, self._value, value,
                 self._descriptor.unit_of_measurement or ''
             )
-            changed = True
-            self._value = value
+            should_notify = True
+            if self._value is None:
+                log_debug("Metric updated to non-None value. forcing it. metric: %s", self.unique_id)
+                force = True
         else:
             log_debug(
                 "Metric %s value unchanged: %s %s",
                 self.unique_id, value,
                 self._descriptor.unit_of_measurement or ''
             )
+        self._value = value
 
-        # In case of zero update frequency, always consider changed when MQTT message is received
-        if self._hub._update_frequency_seconds == 0:
-            changed = True
         # In case of non-zero update frequency, respect the update frequency limit only for numerical values
-        elif self._hub._update_frequency_seconds is not None and isinstance(value, (float, int)):
-            elapsed = now - self._last_update
+        if not force and self._hub._update_frequency_seconds is not None and isinstance(value, (float, int)):
+            elapsed = now - self._last_notified
             if elapsed < self._hub._update_frequency_seconds:
                 _LOGGER.debug(
                     "Update for %s skipped due to frequency limit (%.2fs < %ds)",
                     self.unique_id, elapsed, self._hub._update_frequency_seconds
                 )
-                changed = False
+                should_notify = False
             else:
-                changed = True
+                # This happens when the last time was before the update frequency passed so now we do really need to notify on it
+                should_notify = True
 
-        if changed and event_loop and callable(self._on_update) and event_loop.is_running():
-            self._last_update = now
+        if should_notify and  self._hub._loop and callable(self._on_update) and self._hub._loop.is_running():
+            self._last_notified = now
             try:
                 # If the event loop is running, schedule the callback
-                event_loop.call_soon_threadsafe(self._on_update, self, self.value)
+                self._hub._loop.call_soon_threadsafe(self._on_update, self, self.value)
             except Exception as exc:
                 log_debug("Error calling callback %s", exc, exc_info=True)
 
         for dependency in self._depend_on_me:
             assert self != dependency, f"Circular dependency detected: {self}"
-            dependency._handle_formula(event_loop, log_debug)
+            dependency._handle_formula(log_debug)
